#!/usr/bin/env python
# Replace all sector data with inputs from an image file.
#
import sys
import argparse
import logging
import imagedisk

def generate_imd_fn(fn: str) -> str:
    """ Create new IMD name from given input
        Files starting with '.' don't have a suffix """
    lastPoint = fn.rfind('.')
    if lastPoint <= 0:
        return fn + '.imd'
    else:
        return fn[:lastPoint] + '.imd'

if __name__ == '__main__':
    log_level=logging.INFO
    parser = argparse.ArgumentParser(description='Convert raw files to ImageDisk from template.')
    parser.add_argument('files', metavar='filenames', type=str, nargs='*',
                        help='raw files to convert')
    parser.add_argument('-k', '--keep', default='',
                        help='Keep bad and unavailable blocks from template',
                        action="store_true")
    parser.add_argument('-o', '--output', metavar='filename', default='', help='output file')
    parser.add_argument('-t', '--template', help='IMD file to be used as template')

    parser.add_argument("-d", "--debug", help="log at debug level",
                        action="store_true")
    args = parser.parse_args()
    if args.debug:
        log_level=logging.DEBUG
    logging.basicConfig(level=log_level)
    if args.template is None:
        print("The template argument is mandatory")
        sys.exit(1)
    if len(args.files) == 0:
        print("No files to convert")
        sys.exit(0)
    for fn in args.files:
        template_disk = imagedisk.FloppyImage()
        imagedisk.imdload(template_disk, args.template)
        with open(fn, "rb") as fd1:
            img = fd1.read()
        if template_disk.get_capacity() != len(img):
            print("Mismatch in disk size: raw file must be {} bytes"
                .format(template_disk.get_capacity()))
            sys.exit(1)
        if args.output == '':
            output_name = generate_imd_fn(fn)
        else:
            output_name = args.output
        if output_name.casefold() == args.template.casefold():
            print("Can't save to same name as template file")
            sys.exit(1)
        offset = 0
        for t in template_disk.tracks:
            for sNum in sorted(t.get_sector_list()):
                newdata = img[offset:offset + t.sector_size]
                offset += t.sector_size
                template_disk.changeData(t.head, t.cyl, sNum, newdata)
        writer = imagedisk.IMDWriter(output_name, template_disk.comment)
        template_disk.save(writer)
        writer.close()

